;
; HUC_BRAM.ASM  -  HuC Backup RAM Library
;

		.bss
_bm_mpr4:	.ds 1
_bm_error:	.ds 1
		.code

; bm_check()
; ---

_bm_check:
	  sei
	  tma	#4
	  sta	_bm_mpr4
	  lda	#$F7
	  tam	#4
	  csl
	  lda	#$80
	  sta	bram_unlock
	  ; -- swap bits
	  ldx	#7
.l1:	  lda	$8000,X
	  eor	#$FF
	  sta	<_ax,X
	  sta	$8000,X
	  dex
	  bpl	.l1
	  ; -- cmp
	  cly
	  ldx	#7
.l2:	  lda	<_ax,X
	  cmp	$8000,X
	  beq	.l3
	  iny
.l3:	  eor	#$FF
	  sta	$8000,X
	  dex
	  bpl	.l2
	  ; -- result
	  jsr	_bm_disable
	  cpy	#0
	  bne	.err
	  ; -- ok
	  ldx	#1
	  cla
	  clc
	  rts
	  ; -- no bram
.err:	  clx
	  cla
	  sec
	  rts

; bm_format()
; ---

_bm_format:
	  jsr	_bm_enable
	  bcc	.ok
	  ; -- format
	  ldx	#7
.l1:	  lda	_bm_id,X
	  sta	$8000,X
	  dex
	  bpl	.l1
	  stz	$8010
	  stz	$8011
	  ; -- ok
.ok:	  jsr	_bm_disable
	  stz	_bm_error
	  clx
	  cla
	  clc
	  rts

; bm_free()
; ---

_bm_free:
	  jsr	_bm_enable
	  bcs	.err
	  ; -- calculate free space
	  stw	$8004,<_cx
	  subw	$8006,<_cx
	  subw	#$12,<_cx
	  lda	<_ch
	  bpl	.ok
	  stwz	<_cx
	  ; -- ok
.ok:	  jsr	_bm_disable
	  stz	_bm_error
	__ldw	<_cx
	  clc
	  rts
	  ; -- error, bram not formated
.err:	  sta	_bm_error
	  jsr	_bm_disable
	  lda	#$FF
	  tax
	  sec
	  rts

; bm_exist(char *name)
; ---

_bm_exist:
	__stw	<_bx
	  jsr	_bm_open
	  bcs	.l1
	  jsr	_bm_disable
.l1:	  ldx	_bm_error
	  cla
	  rts

; bm_read(char *buf [di], char *name [bx], int offset [bp], int nb)
; ---

_bm_read.4:
	__stw	<_ax
	  ; -- open file
	  jsr	_bm_open
	  bcs	.x2
	  ; -- checksum test
	  jsr	_bm_checksum
	  ldy	#2
	  lda	[_si],Y
	  add	<_dl
	  sta	<_dl
	  iny
	  lda	[_si],Y
	  adc	<_dh
	  ora	<_dl
	  bne	.x1
	  ; -- setup ptr
	  jsr	_bm_setup_ptr
	  bcs	.ok
	  ; -- read
	  cly
.l1:	  lda	[_dx],Y
	  sta	[_di],Y
	  iny
	  bne	.l2
	  inc	<_dx+1
	  inc	<_di+1
.l2:	  dec	<_cl
	  bne	.l1
	  dec	<_ch
	  bpl	.l1
	  ; -- ok
.ok:	  jsr	_bm_disable
	  stz	_bm_error
	__ldw	<_ax
	  clc
	  rts
	  ; -- error, bad file checksum
.x1:	  lda	#2
	  sta	_bm_error
	  jsr	_bm_disable
	  clx
	  cla
	  sec
.x2:	  rts

; bm_write(char *buf [di], char *name [bx], int offset [bp], int nb)
; ---

_bm_write.4:
	__stw	<_ax
	  ; -- open file
	  jsr	_bm_open
	  bcs	.x1
	  ; -- setup ptr
	  jsr	_bm_setup_ptr
	  bcs	.ok
	  ; -- write data
	  cly
.l1:	  lda	[_di],Y
	  sta	[_dx],Y
	  iny
	  bne	.l2
	  inc	<_dx+1
	  inc	<_di+1
.l2:	  dec	<_cl
	  bne	.l1
	  dec	<_ch
	  bpl	.l1
	  ; -- update checksum
	  jsr	_bm_checksum
	  ldy	#2
	  cla
	  sub	<_dl
	  sta	[_si],Y
	  iny
	  cla
	  sbc	<_dh
	  sta	[_si],Y
	  ; -- ok
.ok:	  jsr	_bm_disable
	  stz	_bm_error
	__ldw	<_ax
	  clc
.x1:	  rts

; bm_create(char *name [bx], int size)
; ---

_bm_create.2:
	__stw	<_ax
	  jsr	_bm_enable
	  bcc	.go
	  bra	.x2
	  ; -- error, not enough ram
.x1:	  lda	#5
	  ; -- error, bram not formated
.x2:	  sta	_bm_error
	  jsr	_bm_disable
	  ldx	_bm_error
	  cla
	  sec
	  rts
	  ; -- check free space
.go:	  addw	#$12,$8006,<_dx
	  addw	<_ax,<_dx
	  cmpw	<_dx,$8004
	  blo	.x1
	  ; -- create file
	  stw	$8006,<_si
	  ldy	#1
	  lda	<_al
	  add	#$10
	  sta	[_si]
	  lda	<_ah
	  adc	#$00
	  sta	[_si],Y
	  ; --
	  lda	$8006
	  add	[_si]
	  sta	<_dl
	  sta	$8006
	  lda	$8007
	  adc	[_si],Y
	  sta	<_dh
	  sta	$8007
	  cla
	  sta	[_dx]
	  sta	[_dx],Y
	  ; -- copy name
	  clx
	  ldy	#4
.l1:	  sxy
	  lda	[_bx],Y
	  sxy
	  sta	[_si],Y
	  iny
	  inx
	  cpx	#12
	  bne	.l1
	  ; -- clear file
	  lda	<_al
	  ora	<_ah
	  beq	.sum
	  stw	<_si,<_bx
	  ldy	#16
	  cla
.l2:	  sta	[_bx],Y
	  iny
	  bne	.l3
	  inc	<_bh
.l3:	  dec	<_al
	  bne	.l2
	  dec	<_ah
	  bpl	.l2
	  ; -- update checksum
.sum:	  jsr	_bm_checksum
	  ldy	#2
	  cla
	  sub	<_dl
	  sta	[_si],Y
	  iny
	  cla
	  sbc	<_dh
	  sta	[_si],Y
	  ; -- ok
.ok:	  jsr	_bm_disable
	  stz	_bm_error
	  clx
	  cla
	  clc
	  rts

; bm_errno()
; ---

_bm_errno:
	  ldx	_bm_error
	  cla
	  rts

; bm_open(char *name [bx])
; ---

_bm_open:
	  jsr	_bm_enable
	  bcs	.x2
	  ; -- get dir entry
	  stw	#$8010,<_si
.l1:	  lda	[_si]
	  sta	<_cl
	  ldy	#1
	  lda	[_si],Y
	  sta	<_ch
	  ora	<_cl
	  beq	.x1
	  addw	<_cx,<_si,<_dx
	  cmpw	<_dx,$8004
	  blo	.x3
	  cmpw	#16,<_cx
	  blo	.x3
	  ; -- compare names
	  cly
	  ldx	#4
.l2:	  lda	[_bx],Y
	  sxy
	  cmp	[_si],Y
	  bne	.next
	  sxy
	  inx
	  iny
	  cpy	#12
	  blo	.l2
	  ; -- check file size
	  lda	<_ch
	  bne	.ok
	  lda	<_cl
	  cmp	#16
	  beq	.x4
	  ; -- ok
.ok:	  stz	_bm_error
	  clc
	  rts
	  ; -- next entry
.next:	  addw	<_cx,<_si
	  bra	.l1
	  ; -- error, file not found
.x1:	  lda	#1
	  ; -- error, bram not formated
.x2:	  sta	_bm_error
	  jsr	_bm_disable
	  sec
	  rts
	  ; -- error, directory corrupted
.x3:	  lda	#3
	  bra	.x2
	  ; -- error, empty file
.x4:	  lda	#4
	  bra	.x2

; bm_enable()
; ---

_bm_id:
	  .db	$48,$55,$42,$4D
	  .db	$00,$88,$10,$80
_bm_enable:
	  sei
	  tma	#4
	  sta	_bm_mpr4
	  lda	#$F7
	  tam	#4
	  csl
	  lda	#$80
	  sta	bram_unlock
	  ; -- check if formated
	  ldx	#3
.l1:	  lda	$8000,X
	  cmp	_bm_id,X
	  bne	.x1
	  dex
	  bpl	.l1
	  ; -- ok
	  cla
	  clc
	  rts
	  ; -- error, not formated!
.x1:	  lda	#$FF
	  sec
	  rts

; bm_disable()
; ---

_bm_disable:
	  lda	_bm_mpr4
	  tam	#4
	  lda	bram_lock
	  csh
	  cli
	  rts

; bm_checksum(char *fcb [si])
; ---

_bm_checksum:
	  stwz	<_dx
	  ; -- get file size
	  lda	[_si]
	  sub	#4
	  sta	<_cl
	  ldy	#1
	  lda	[_si],Y
	  sbc	#0
	  sta	<_ch
	  stw	<_si,<_bx
	  ; -- calc checksum
	  ldy	#4
.l1:	  lda	[_bx],Y
	  add	<_dl
	  sta	<_dl
	  bcc	.l2
	  inc	<_dh
.l2:	  iny
	  bne	.l3
	  inc	<_bh
.l3:	  dec	<_cl
	  bne	.l1
	  dec	<_ch
	  bpl	.l1
	  rts

; bm_setup_ptr(char *fcb [si], char *buf [di], int offset [bp], int nb [ax])
; ---

_bm_setup_ptr:
	  ; -- check length
	  tstw	<_ax
	  beq	.x1
	  ; -- check ptr
	  tstw	<_di
	  beq	.x1
	  ; -- check offset
	  addw	#16,<_bp,<_bx
	  ldy	#1
	  lda	<_bh
	  cmp	[_si],Y
	  bne	.l1
	  lda	<_bl
	  cmp	[_si]
.l1:	  blo	.l2
	  ; -- eof
.x1:	  stwz	<_ax
	  sec
	  rts
	  ; -- set base ptr
.l2:	  addw	<_bx,<_si,<_dx
	  ; -- check length
	  addw	<_ax,<_bx
	  lda	[_si]
	  sub	<_bl
	  sta	<_bl
	  lda	[_si],Y
	  sbc	<_bh
	  sta	<_bh
	  bpl	.ok
	  ; -- adjust size
	  addw	<_bx,<_ax
.ok:	  stw	<_ax,<_cx
	  clc
	  rts

